import java.awt.Color;
import java.io.FileOutputStream;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import com.arctorus.documents.BaselineType;
import com.arctorus.documents.Document;
import com.arctorus.documents.ExportType;
import com.arctorus.documents.FillFormat;
import com.arctorus.documents.FillFormatType;
import com.arctorus.documents.HorizontalAlignmentType;
import com.arctorus.documents.Page;
import com.arctorus.documents.Run;
import com.arctorus.documents.TextFormat;
import com.arctorus.documents.VerticalAlignmentType;
import com.arctorus.documents.table.Cell;
import com.arctorus.documents.table.CellStyle;
import com.arctorus.documents.table.CellStyleManager;
import com.arctorus.documents.table.MergedCellException;
import com.arctorus.documents.table.Row;
import com.arctorus.documents.table.Table;

public class InvoiceReport1 {

    private static final String QUERY =
        "SELECT DISTINCTROW Invoices.ShipCountry, Invoices.ShipCity, Invoices.ProductName, "
            + "Customers.ContactName, Customers.CompanyName "
            + "FROM Invoices, Customers WHERE Invoices.CustomerID = Customers.CustomerID "
            + "ORDER BY Invoices.ShipCountry, Invoices.ShipCity, Customers.CompanyName, Customers.ContactName";

    private static final String MAIN_HEADER = "List of Invoices Grouped by Countries";

    private static final String[] TABLE_COLUMNS = new String[] {
        "Ship City:", "Product Name:", "Contact Name", "Company Name:"
    };

    public InvoiceReport1() {
    }

    public void createReport(Statement statement, String outputPath) throws Exception {

        Document xlsxDoc = new Document();
        Page page = xlsxDoc.getFirstPage();

        ResultSet resultSet = statement.executeQuery(QUERY);
        fillInvoicesPage(page, resultSet, xlsxDoc);
        resultSet.close();

        xlsxDoc.save(ExportType.XLSX, new FileOutputStream(outputPath + "invoices1.xlsx"));
        xlsxDoc.save(ExportType.ODS, new FileOutputStream(outputPath + "invoices1.ods"));
    }

    private static void fillInvoicesPage(Page page, ResultSet resultSet, Document doc) throws Exception {

        Table table = page.getTable();

        // Set columns width
        table.getColumn("A").setWidth(150);
        table.getColumn("B").setWidth(250);
        table.getColumn("C").setWidth(140);
        table.getColumn("D").setWidth(180);

        int nextRow = 1;
        nextRow = drawHeader(table, nextRow, MAIN_HEADER);
        String oldCountry = " ";
        boolean oddRow = true;
        while (resultSet.next()) {
            String country = resultSet.getString(1);
            if (!country.equals(oldCountry)) {
                oldCountry = country;
                nextRow = drawCountryHeader(doc, table, nextRow, country);
                oddRow = true;
            }

            drawInvoice(doc, table, nextRow++, oddRow, resultSet.getString(2), resultSet.getString(3),
                resultSet.getString(4), resultSet.getString(5));
            oddRow = !oddRow;
        }
    }

    private static int drawHeader(Table table, int nextRow, String headerText) throws Exception {

        // Create text format for the header
        TextFormat txtFormat = new TextFormat("Arial", (short) 14);
        txtFormat.setBold(true);

        // Little increase row height.
        Row row = table.getRow(nextRow);
        row.setHeight(row.getHeight() * 1.3f);

        // Create text block and add it to the cell
        Run run = new Run(headerText);
        run.setTextFormat(txtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        ++nextRow;

        // Create text format for the Date field
        txtFormat = new TextFormat("Arial", (short) 11);

        // Convert date to string
        DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy", Locale.ENGLISH);
        String s = formatter.format(new Date());

        // Add date to the cell
        run = new Run(s);
        run.setTextFormat(txtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        return ++nextRow;
    }

    private static int drawCountryHeader(Document document, Table table,
            int nextRow, String country) throws NullPointerException, MergedCellException {

        CellStyleManager csManager = document.getCellStyleManager();

        nextRow++;

        // Create text format for the cells with country.
        TextFormat txtFormat = new TextFormat("Arial", (short) 14);
        txtFormat.setBaseline(BaselineType.NORMAL);
        txtFormat.setColor(new Color(0, 51, 102));
        txtFormat.setBold(true);
        txtFormat.setItalic(true);

        // Text should be aligned left in the cells
        CellStyle cStyle = csManager.findCellStyle(new FillFormat(),
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.CENTER, false, false);

        // Little increase row height.
        Row row = table.getRow(nextRow);
        row.setHeight(row.getHeight() * 1.3f);
        
        // Create text in the cell and assign cell styles
        Cell cell = table.getCell(0, nextRow);
        cell.setStyleId(cStyle.getId());
        Run run = new Run(country.toUpperCase());
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);
        ++nextRow;

        // Create text format for the sub-header.
        txtFormat = new TextFormat("Arial", (short) 11);
        txtFormat.setBold(true);
        txtFormat.setUnderline(true);

        // Cell style with right aligned text
        cStyle = csManager.findCellStyle(new FillFormat(),
            HorizontalAlignmentType.RIGHT, VerticalAlignmentType.TOP, false, false);

        // Create fill format for the cells with header.
        FillFormat ffH = new FillFormat();
        ffH.setType(FillFormatType.SOLID);
        //ffH.setColor(new Color(79, 129, 189));
        ffH.setColor(new Color(107, 174, 255));

        // Cell style with left aligned text
        CellStyle cStyleH = csManager.findCellStyle(ffH,
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.CENTER, false, false);

        // Fill sub-header cells with text.
        for (int i = 0; i < 4; i++) {
            cell = table.getCell(i, nextRow);
            cell.setStyleId(cStyleH.getId());
            run = new Run(TABLE_COLUMNS[i]);
            run.setTextFormat(txtFormat);
            cell.getRuns().add(run);
        }
        return ++nextRow;
    }

    private static void drawInvoice(Document document, Table table, int nextRow, boolean oddRow,
            String contactName, String companyName, String productName, String shipCity) throws Exception {

        CellStyleManager csManager = document.getCellStyleManager();

        FillFormat ff = new FillFormat();
        ff.setType(FillFormatType.SOLID);

        // Set different colors for even and odd rows
        if (oddRow)
            ff.setColor(new Color(174, 211, 255));
        else
            ff.setColor(new Color(216, 234, 255));

        CellStyle cStyle = csManager.findCellStyle(ff,
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.CENTER, false, false);

        // Create text format for all product description cells
        TextFormat txtFormat = new TextFormat("Arial", (short) 11);

        // Write "Contact Name" to a cell
        Cell cell = table.getCell(0, nextRow);
        cell.setStyleId(cStyle.getId());
        Run run = new Run(contactName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Write "Company Name" to a cell
        cell = table.getCell(1, nextRow);
        cell.setStyleId(cStyle.getId());
        run = new Run(companyName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Write "Product Name" to a cell
        cell = table.getCell(2, nextRow);
        cell.setStyleId(cStyle.getId());
        run = new Run(productName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Write "Ship City" to a cell
        cell = table.getCell(3, nextRow);
        cell.setStyleId(cStyle.getId());
        run = new Run(shipCity);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);
    }
}
