import java.awt.Color;
import java.io.FileOutputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import com.arctorus.documents.Document;
import com.arctorus.documents.ExportType;
import com.arctorus.documents.FillFormat;
import com.arctorus.documents.FillFormatType;
import com.arctorus.documents.HorizontalAlignmentType;
import com.arctorus.documents.Page;
import com.arctorus.documents.Run;
import com.arctorus.documents.StrokeFormat;
import com.arctorus.documents.StrokeFormatType;
import com.arctorus.documents.TextFormat;
import com.arctorus.documents.VerticalAlignmentType;
import com.arctorus.documents.table.BorderStyle;
import com.arctorus.documents.table.BorderStyleManager;
import com.arctorus.documents.table.Cell;
import com.arctorus.documents.table.CellStyle;
import com.arctorus.documents.table.CellStyleManager;
import com.arctorus.documents.table.MergedCell;
import com.arctorus.documents.table.MergedCellException;
import com.arctorus.documents.table.Row;
import com.arctorus.documents.table.Table;

public class InvoiceReport2 {

    private static final String QUERY =
        "SELECT DISTINCTROW "
            + "ShipName, ShipAddress, ShipCity, ShipRegion, ShipPostalCode, ShipCountry, "
            + "Customers.CompanyName, Address, City, Region, PostalCode, Country, "
            + "OrderID, CustomerID, Salesperson, OrderDate, RequiredDate, ShippedDate, "
            + "Shippers.CompanyName as ShippersCompanyName, "
            + "ProductID, ProductName, Quantity, UnitPrice, Discount, ExtendedPrice "
            + "FROM Invoices ORDER BY OrderID, ProductID ";

    private static final String MAIN_HEADER = "INVOICE";
    private static final String SHIP_TO_HEADER = "Ship To:";
    private static final String BILL_TO_HEADER = "Bill To:";

    private static final String[] ORDER_TITLES = new String[] {
        "Order ID:", "Customer ID:", "Salesperson:", "Order Date:", "Required Date:", "Shipped Date:", "Ship Via:"
    };

    private static final String[] ORDER_COLUMN_NAMES = new String[] {
        "OrderID", "CustomerID", "Salesperson", "OrderDate", "RequiredDate", "ShippedDate", "ShippersCompanyName"
    };

    private static final String[] PRODUCT_TITLES = new String[] {
        "Product ID:", "Product Name:", "Quantity:", "Unit Price:", "Discount:", "Extended Price:"
    };

    private static final String[] PRODUCT_COLUMN_NAMES = new String[] {
        "ProductID", "ProductName", "Quantity", "UnitPrice", "Discount", "ExtendedPrice"
    };

    public InvoiceReport2() {
    }

    public void createReport(Statement statement, String outputPath) throws Exception {

        Document xlsxDoc = new Document();
        Page page = xlsxDoc.getFirstPage();

        int startRow = 1;
        for (int i = 0; i < 1; i++) {
        ResultSet resultSet = statement.executeQuery(QUERY);
            startRow = fillOrdersPage(page, resultSet, xlsxDoc, startRow + 1);
        resultSet.close();
        }

        xlsxDoc.save(ExportType.XLSX, new FileOutputStream(outputPath + "invoices2.xlsx"));
        xlsxDoc.save(ExportType.ODS, new FileOutputStream(outputPath + "invoices2.ods"));
    }

    private int fillOrdersPage(Page page, ResultSet resultSet, Document doc, int startRow) throws Exception {

        Table table = page.getTable();

        // Set columns width
        table.getColumn("A").setWidth(70);
        table.getColumn("B").setWidth(100);
        table.getColumn("C").setWidth(100);
        table.getColumn("D").setWidth(100);
        table.getColumn("E").setWidth(100);
        table.getColumn("F").setWidth(100);
        table.getColumn("G").setWidth(100);

        int nextRow = startRow;

        int oldOrderID = 0;
        while (resultSet.next()) {
            int orderID = resultSet.getInt("OrderID");
            if (orderID != oldOrderID) {
                oldOrderID = orderID;
                nextRow += 3;
                nextRow = drawHeader(doc, table, nextRow, MAIN_HEADER);
                nextRow = drawShipHeader(doc, table, nextRow, resultSet);
                nextRow = drawOrderInfo(doc, table, orderID, nextRow++, resultSet);
                nextRow = drawProductHeader(doc, table, nextRow++);
            }
            nextRow = drawProduct(doc, table, nextRow++, resultSet);
        }
        
        return nextRow;
    }

    private int drawProduct(Document doc, Table table, int nextRow, ResultSet resultSet) throws SQLException {

        CellStyleManager csManager = doc.getCellStyleManager();

        // Create text format for the cells with product data.
        TextFormat txtFormatVal = new TextFormat("Arial", (short) 10);
        txtFormatVal.setColor(new Color(0, 0, 0));

        // Create fill format for the cells with product header.
        FillFormat ffVal = new FillFormat();
        ffVal.setType(FillFormatType.NO_FILL);

        // Cell style with left aligned text for string data
        CellStyle cStyleStrVal = csManager.findCellStyle(ffVal,
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.CENTER, false, false);

        // Cell style with right aligned text for value data
        CellStyle cStyleVal = csManager.findCellStyle(ffVal,
            HorizontalAlignmentType.RIGHT, VerticalAlignmentType.CENTER, false, false);

        // Prepare formatter for double values
        NumberFormat formatter = new DecimalFormat("##0.##");

        for (int i = 0; i < PRODUCT_COLUMN_NAMES.length; i++) {

            Cell cell;
            if (i < 2)
                cell = table.getCell(i, nextRow);
            else
                cell = table.getCell(i + 1, nextRow);

            Run run = null;
            if (PRODUCT_COLUMN_NAMES[i].equals("ProductName")) {
                // Fill string values
                cell.setStyleId(cStyleStrVal.getId());
                run = new Run(resultSet.getString(PRODUCT_COLUMN_NAMES[i]));
            } else if (PRODUCT_COLUMN_NAMES[i].equals("Quantity") || PRODUCT_COLUMN_NAMES[i].equals("ProductID")) {
                // Fill integer values
                cell.setStyleId(cStyleVal.getId());
                String val = String.valueOf(resultSet.getInt(PRODUCT_COLUMN_NAMES[i]));
                run = new Run(val);
            } else {
                // Fill double values
                cell.setStyleId(cStyleVal.getId());
                String val = formatter.format(resultSet.getDouble(PRODUCT_COLUMN_NAMES[i]));
                // Add "%" for Discount
                if (PRODUCT_COLUMN_NAMES[i].equals("Discount"))
                    val += "%";
                run = new Run(val);
            }
            if (run != null) {
                run.setTextFormat(txtFormatVal);
                cell.getRuns().add(run);
            }
        }
        nextRow++;
        return nextRow;
    }

    private int drawProductHeader(Document doc, Table table, int nextRow) {

        CellStyleManager csManager = doc.getCellStyleManager();

        // Create text format for the cells with product header.
        TextFormat txtFormat = new TextFormat("Arial", (short) 10);
        txtFormat.setColor(new Color(255, 255, 255));
        txtFormat.setBold(true);

        // Create fill format for the cells with product header.
        FillFormat ff = new FillFormat();
        ff.setType(FillFormatType.SOLID);
        ff.setColor(new Color(79, 189, 129));

        // Cell style with left aligned text
        CellStyle cStyle1 = csManager.findCellStyle(ff,
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.CENTER, false, false);
        CellStyle cStyle2 = csManager.findCellStyle(ff,
            HorizontalAlignmentType.RIGHT, VerticalAlignmentType.CENTER, false, false);

        try {
            MergedCell mergedCell = new MergedCell(table.getCell(1, nextRow), table.getCell(2, nextRow));
            table.addMergedCell(mergedCell);
        } catch (NullPointerException e) {
        } catch (MergedCellException e) {
        }
        
        // Fill sub-header cells with text.
        for (int i = 0; i < PRODUCT_TITLES.length; i++) {
            Cell cell;
            if (i < 2) {
                cell = table.getCell(i, nextRow);
                cell.setStyleId(cStyle1.getId());
            } else {
                cell = table.getCell(i + 1, nextRow);
                cell.setStyleId(cStyle2.getId());
            }
            Run run = new Run(PRODUCT_TITLES[i]);
            run.setTextFormat(txtFormat);
            cell.getRuns().add(run);
        }
        nextRow++;
        return nextRow;
    }

    private int drawOrderInfo(Document doc, Table table, int orderID,
            int nextRow, ResultSet resultSet) throws SQLException {

        CellStyleManager csManager = doc.getCellStyleManager();

        // Create text format for the cells with order header.
        TextFormat txtFormat = new TextFormat("Arial", (short) 10);
        txtFormat.setColor(new Color(255, 255, 255));
        txtFormat.setBold(true);

        // Create fill format for the cells with order header.
        FillFormat ff = new FillFormat();
        ff.setType(FillFormatType.SOLID);
        ff.setColor(new Color(79, 189, 129));

        // Cell style with left aligned text
        CellStyle cStyle = csManager.findCellStyle(ff,
            HorizontalAlignmentType.CENTER, VerticalAlignmentType.CENTER, false, false);

        // Fill sub-header cells with text.
        for (int i = 0; i < ORDER_TITLES.length; i++) {
            Cell cell = table.getCell(i, nextRow);
            cell.setStyleId(cStyle.getId());
            Run run = new Run(ORDER_TITLES[i]);
            run.setTextFormat(txtFormat);
            cell.getRuns().add(run);
        }
        nextRow++;

        // Create text format for the cells with order data.
        TextFormat txtFormatVal = new TextFormat("Arial", (short) 10);
        txtFormatVal.setColor(new Color(0, 0, 0));

        // Create fill format for the cells with order data.
        FillFormat ffVal = new FillFormat();
        ffVal.setType(FillFormatType.NO_FILL);

        // Cell style with left aligned text
        CellStyle cStyleVal = csManager.findCellStyle(ffVal,
            HorizontalAlignmentType.CENTER, VerticalAlignmentType.CENTER, false, false);

        // Prepare date formatter
        DateFormat formatter = new SimpleDateFormat("MMM dd, yyyy", Locale.ENGLISH);

        for (int i = 0; i < ORDER_COLUMN_NAMES.length; i++) {
            Cell cell = table.getCell(i, nextRow);
            cell.setStyleId(cStyleVal.getId());
            Run run = null;
            if (ORDER_COLUMN_NAMES[i] == "OrderID") {
                // Fill orderId value
                run = new Run(String.valueOf(orderID));
            } else if (ORDER_COLUMN_NAMES[i].endsWith("Date")) {
                // Fill order data cells with dates
                Date date = resultSet.getDate(ORDER_COLUMN_NAMES[i]);
                if (date != null)
                    run = new Run(formatter.format(date));
            } else
                // Fill order data cells with strings
                run = new Run(resultSet.getString(ORDER_COLUMN_NAMES[i]));

            if (run != null) {
                run.setTextFormat(txtFormatVal);
                cell.getRuns().add(run);
            }
        }
        nextRow += 2;
        return nextRow;
    }

    private int drawShipHeader(Document doc, Table table, int nextRow, ResultSet resultSet) throws SQLException {

        // Create text format for the header
        TextFormat txtFormat = new TextFormat("Arial", (short) 12);
        txtFormat.setColor(new Color(0, 0, 159));
        txtFormat.setBold(true);

        // Create text block and add it to the cell
        Run runShip = new Run(SHIP_TO_HEADER);
        runShip.setTextFormat(txtFormat);
        table.getCell(0, nextRow).getRuns().add(runShip);

        // Create text block and add it to the cell
        Run runBill = new Run(BILL_TO_HEADER);
        runBill.setTextFormat(txtFormat);
        table.getCell(4, nextRow).getRuns().add(runBill);

        // Create text format for the header
        TextFormat simpleTxtFormat = new TextFormat("Arial", (short) 10);
        simpleTxtFormat.setColor(Color.BLACK);

        // Write "Ship Name" to a cell
        Run run = new Run(resultSet.getString("ShipName"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        // Write "Company Name" to a cell
        run = new Run(resultSet.getString("CompanyName"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(5, nextRow).getRuns().add(run);

        nextRow++;

        // Write "Ship Address" to a cell
        run = new Run(resultSet.getString("ShipAddress"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        // Write "Address" to a cell
        run = new Run(resultSet.getString("Address"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(5, nextRow).getRuns().add(run);

        nextRow++;

        // Prepare and write "ShipCity, ShipRegion, ShipPostalCode" string to
        // cell
        String addr = "";
        String tmp = resultSet.getString("ShipCity");
        if (tmp != null && tmp.length() > 0) {
            addr += tmp;
            addr += ", ";
        }
        tmp = resultSet.getString("ShipRegion");
        if (tmp != null && tmp.length() > 0) {
            addr += tmp;
            addr += ", ";
        }
        tmp = resultSet.getString("ShipPostalCode");
        if (tmp != null && tmp.length() > 0)
            addr += tmp;
        run = new Run(addr);
        run.setTextFormat(simpleTxtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        // Prepare and write "City, Region, PostalCode" string to cell
        addr = "";
        tmp = resultSet.getString("City");
        if (tmp != null && tmp.length() > 0) {
            addr += tmp;
            addr += ", ";
        }
        tmp = resultSet.getString("Region");
        if (tmp != null && tmp.length() > 0) {
            addr += tmp;
            addr += ", ";
        }
        tmp = resultSet.getString("PostalCode");
        if (tmp != null && tmp.length() > 0)
            addr += tmp;
        run = new Run(addr);
        run.setTextFormat(simpleTxtFormat);
        table.getCell(5, nextRow).getRuns().add(run);

        nextRow++;

        // Write "Ship Country" to a cell
        run = new Run(resultSet.getString("ShipCountry"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        // Write "Country" to a cell
        run = new Run(resultSet.getString("Country"));
        run.setTextFormat(simpleTxtFormat);
        table.getCell(5, nextRow).getRuns().add(run);

        nextRow += 2;
        return nextRow;
    }

    private static int drawHeader(Document doc, Table table, int nextRow, String headerText) throws Exception {

        // Create text format for the header
        TextFormat txtFormat = new TextFormat("Arial", (short) 14);
        txtFormat.setColor(new Color(0, 0, 159));
        txtFormat.setBold(true);

        // Little increase row height.
        Row row = table.getRow(nextRow);
        row.setHeight(row.getHeight() * 1.3f);

        // Create text block and add it to the cell
        Run run = new Run(headerText);
        run.setTextFormat(txtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        ++nextRow;

        BorderStyleManager bsManager = doc.getBorderStyleManager();

        // Create Stroke format and border style to draw horizontal line
        StrokeFormat stroke = new StrokeFormat();
        stroke.setColor(Color.BLACK);
        stroke.setType(StrokeFormatType.SOLID);
        stroke.setWidth(2);

        BorderStyle bStyle = bsManager.findBorderStyle(stroke);

        // Add Top borders for all cells in a row
        nextRow++;
        for (int i = 0; i < 7; i++) {
            table.getCell(i, nextRow).getBorders().setTopBorder(bStyle.getId());
        }

        nextRow++;
        // Create text format for the Date field
        txtFormat = new TextFormat("Arial", (short) 11);

        // Convert date to string
        DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy", Locale.ENGLISH);
        String s = formatter.format(new Date());

        // Add date to the cell
        run = new Run(s);
        run.setTextFormat(txtFormat);
        table.getCell(0, nextRow).getRuns().add(run);

        nextRow += 2;

        return nextRow;
    }
}
