import java.awt.Color;
import java.io.FileOutputStream;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import com.arctorus.documents.BaselineType;
import com.arctorus.documents.Document;
import com.arctorus.documents.ExportType;
import com.arctorus.documents.FillFormat;
import com.arctorus.documents.FillFormatType;
import com.arctorus.documents.HorizontalAlignmentType;
import com.arctorus.documents.Page;
import com.arctorus.documents.Run;
import com.arctorus.documents.StrokeFormat;
import com.arctorus.documents.StrokeFormatType;
import com.arctorus.documents.TextFormat;
import com.arctorus.documents.VerticalAlignmentType;
import com.arctorus.documents.table.BorderStyle;
import com.arctorus.documents.table.BorderStyleManager;
import com.arctorus.documents.table.Cell;
import com.arctorus.documents.table.CellStyle;
import com.arctorus.documents.table.CellStyleManager;
import com.arctorus.documents.table.Row;
import com.arctorus.documents.table.Table;

public class ProductReport {

    private static final String MAIN_HEADER = "Alphabetical List of Products";

    private static final String QUERY =
        "SELECT DISTINCTROW Products.ProductName, Categories.CategoryName, Products.QuantityPerUnit, Products.UnitsInStock "
            + "FROM Categories INNER JOIN Products ON Categories.CategoryID = Products.CategoryID "
            + "WHERE (((Products.Discontinued) = No)) " + "ORDER BY Products.ProductName";

    private static final String[] TABLE_COLUMNS = new String[] {
        "Product Name", "Category Name:", "Quantity Per Unit:", "Units In Stock:"
    };

    public ProductReport() {
    }

    public void createReport(Statement statement, String outputPath) throws Exception {

        Document xlsxDoc = new Document();
        
        Page page = xlsxDoc.getFirstPage();

        ResultSet resultSet = statement.executeQuery(QUERY);
        fillProductsPage(page, resultSet, xlsxDoc);
        resultSet.close();

        xlsxDoc.save(ExportType.XLSX, new FileOutputStream(outputPath + "products.xlsx"));
        xlsxDoc.save(ExportType.ODS, new FileOutputStream(outputPath + "products.ods"));
    }

    private static void fillProductsPage(Page page, ResultSet resultSet, Document doc) throws Exception {

        Table table = page.getTable();

        // Set columns width
        table.getColumn("A").setWidth(25);
        table.getColumn("B").setWidth(250);
        table.getColumn("C").setWidth(150);
        table.getColumn("D").setWidth(120);
        table.getColumn("E").setWidth(90);

        int nextRow = 1;

        nextRow = drawHeader(table, nextRow, MAIN_HEADER);
        String ch = " ";
        while (resultSet.next()) {
            String productName = resultSet.getString(1);
            if (!productName.startsWith(ch)) {
                ch = productName.substring(0, 1);
                nextRow = drawSubHeader(doc, table, nextRow, ch);
            }
            drawProduct(doc, table, nextRow++, productName,
                resultSet.getString(2), resultSet.getString(3), resultSet.getInt(4));
        }
    }

    private static int drawHeader(Table table, int nextRow, String headerText) throws Exception {
        // Create text format for the header
        TextFormat txtFormat = new TextFormat("Arial", (short) 14);
        txtFormat.setBold(true);

        // Little increase row height.
        Row row = table.getRow(nextRow);
        row.setHeight(row.getHeight() * 1.3f);

        // Create text block and add it to the cell
        Run run = new Run(headerText);
        run.setTextFormat(txtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        ++nextRow;

        // Create text format for the Date field
        txtFormat = new TextFormat("Arial", (short) 11);

        // Convert date to string
        DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy", Locale.ENGLISH);
        String s = formatter.format(new Date());

        // Add date to the cell
        run = new Run(s);
        run.setTextFormat(txtFormat);
        table.getCell(1, nextRow).getRuns().add(run);

        return ++nextRow;
    }

    private static int drawSubHeader(Document document, Table table, int nextRow, String ch) {

        BorderStyleManager bsManager = document.getBorderStyleManager();
        CellStyleManager csManager = document.getCellStyleManager();

        // Create Stroke format and border style to draw horizontal line
        StrokeFormat stroke = new StrokeFormat();
        stroke.setColor(Color.BLACK);
        stroke.setType(StrokeFormatType.SOLID);
        stroke.setWidth(2);

        BorderStyle bStyle = bsManager.findBorderStyle(stroke);

        // Add Top borders for all cells in a row
        nextRow++;
        for (int i = 0; i < 5; i++) {
            table.getCell(i, nextRow).getBorders().setTopBorder(bStyle.getId());
        }

        // Create text format for the cells with alphabet.
        TextFormat txtFormat = new TextFormat("Arial", (short) 12);
        txtFormat.setBaseline(BaselineType.NORMAL);
        txtFormat.setColor(Color.WHITE);
        txtFormat.setBold(true);

        // Create fill format for the cells with alphabet.
        // Cells filled with black solid color.
        FillFormat ff = new FillFormat();
        ff.setType(FillFormatType.SOLID);
        ff.setColor(Color.BLACK);

        // Text should be centered in the cells
        CellStyle cStyle = csManager.findCellStyle(ff,
            HorizontalAlignmentType.CENTER, VerticalAlignmentType.CENTER, false, false);

        // Create text in the cell and assign cell styles
        Cell cell = table.getCell(0, nextRow);
        cell.setStyleId(cStyle.getId());
        Run run = new Run(ch.toUpperCase());
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);
        ++nextRow;

        // Create text format for the sub-header.
        txtFormat = new TextFormat("Arial", (short) 11);
        txtFormat.setBold(true);

        // Cell style with right aligned text
        cStyle = csManager.findCellStyle(new FillFormat(),
            HorizontalAlignmentType.RIGHT, VerticalAlignmentType.TOP, false, false);

        // Fill sub-header cells with text.
        for (int i = 1; i < 5; i++) {
            cell = table.getCell(i, nextRow);
            if (i == 4)
                cell.setStyleId(cStyle.getId());
            run = new Run(TABLE_COLUMNS[i - 1]);
            run.setTextFormat(txtFormat);
            cell.getRuns().add(run);
        }
        return ++nextRow;
    }

    private static void drawProduct(Document document, Table table, int nextRow, String productName,
            String categoryName, String quantityPerUnit, int unitsInStock) throws Exception {

        CellStyleManager csManager = document.getCellStyleManager();

        // Create text format for all product description cells
        TextFormat txtFormat = new TextFormat("Arial", (short) 11);

        // Write "Product Name" to a cell
        Cell cell = table.getCell(1, nextRow);
        Run run = new Run(productName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Write "Category Name" to a cell
        cell = table.getCell(2, nextRow);
        run = new Run(categoryName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Write "Quantity Per Unit" to a cell
        cell = table.getCell(3, nextRow);
        run = new Run(quantityPerUnit);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        // Cell style with right aligned text
        CellStyle cStyle = csManager.findCellStyle(new FillFormat(),
            HorizontalAlignmentType.RIGHT, VerticalAlignmentType.TOP, false, false);

        // Write "Units In Stock" to a cell
        cell = table.getCell(4, nextRow);
        cell.setStyleId(cStyle.getId());
        run = new Run(String.valueOf(unitsInStock));
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);
    }
}
