import java.awt.Color;
import java.awt.Paint;
import java.awt.geom.Rectangle2D;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import com.arctorus.documents.Document;
import com.arctorus.documents.DocumentException;
import com.arctorus.documents.ExportType;
import com.arctorus.documents.FillFormat;
import com.arctorus.documents.FillFormatType;
import com.arctorus.documents.HorizontalAlignmentType;
import com.arctorus.documents.Page;
import com.arctorus.documents.Picture;
import com.arctorus.documents.Run;
import com.arctorus.documents.Shape;
import com.arctorus.documents.TextFormat;
import com.arctorus.documents.VerticalAlignmentType;
import com.arctorus.documents.table.Cell;
import com.arctorus.documents.table.CellStyle;
import com.arctorus.documents.table.CellStyleManager;
import com.arctorus.documents.table.Row;
import com.arctorus.documents.table.Table;

public class SalesReport {

    private static final String MAIN_HEADER = "Sales by Category: ";

    private static final String QUERY =
        "SELECT CategoryName, ProductName, ProductSales " + "FROM [Sales by Category] "
            + "ORDER BY CategoryName, ProductName;";

    private static final String[] TABLE_COLUMNS = new String[] { "Product Name:", "Product Sales:" };

    public SalesReport() {
    }

    public void createReport(Statement statement, String outputPath) throws Exception {

        Document xlsxDoc = new Document();

        ResultSet resultSet = statement.executeQuery(QUERY);
        fillSales(resultSet, xlsxDoc, outputPath);
        resultSet.close();

        xlsxDoc.save(ExportType.XLSX, new FileOutputStream(outputPath + "sales.xlsx"));
        xlsxDoc.save(ExportType.ODS, new FileOutputStream(outputPath + "sales.ods"));
    }

    private static void fillSales(ResultSet resultSet, Document doc, String outputPath) throws Exception {

        Page page = doc.getFirstPage();

        String oldCategory = "";
        Table table = null;
        int nextRow = 1;
        DefaultCategoryDataset dataset = null;
        String category = "";
        while (resultSet.next()) {
            category = resultSet.getString(1);
            if (!category.equals(oldCategory)) {
                if (dataset != null)
                    AddChart(doc, page, nextRow, oldCategory, dataset, outputPath);
                if (oldCategory.length() > 0)
                    page = doc.createPage();

                table = page.getTable();
                // Set columns width
                table.getColumn("A").setWidth(200);
                table.getColumn("B").setWidth(150);

                dataset = new DefaultCategoryDataset();
                oldCategory = category;
                nextRow = 1;
                nextRow = drawHeader(table, nextRow, MAIN_HEADER + category);
                nextRow = drawSubHeader(doc, table, nextRow);
            }

            String productName = resultSet.getString("ProductName");
            Double sales = resultSet.getDouble("ProductSales");
            // Add info for chart
            dataset.addValue(sales, "", productName);

            drawSales(doc, table, nextRow++, productName, sales);
        }
        if (dataset != null && category.length() > 0)
            AddChart(doc, page, nextRow, category, dataset, outputPath);
    }

    private static void drawSales(Document doc, Table table, int nextRow,
            String productName, Double productSales) {

        // Create text format for all sales description cells
        TextFormat txtFormat = new TextFormat("Arial", (short) 11);

        // Write "Category Name" to a cell
        Cell cell = table.getCell(0, nextRow);
        Run run = new Run(productName);
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);

        NumberFormat formatter = new DecimalFormat("##0.##");

        // Write "Sales" to a cell
        cell = table.getCell(1, nextRow);
        run = new Run(formatter.format(productSales));
        run.setTextFormat(txtFormat);
        cell.getRuns().add(run);
    }

    private static int drawSubHeader(Document document, Table table, int nextRow) {

        CellStyleManager csManager = document.getCellStyleManager();

        // Create text format for the sub-header.
        TextFormat txtFormat = new TextFormat("Arial", (short) 10);
        txtFormat.setBold(true);

        // Create fill format for the cells with sales header.
        FillFormat ff = new FillFormat();
        ff.setType(FillFormatType.SOLID);
        ff.setColor(new Color(231, 185, 181));

        // Cell style with left aligned text
        CellStyle cStyle = csManager.findCellStyle(ff,
            HorizontalAlignmentType.LEFT, VerticalAlignmentType.TOP, false, false);

        // Fill sub-header cells with text.
        for (int i = 0; i < TABLE_COLUMNS.length; i++) {
            Cell cell = table.getCell(i, nextRow);
            cell.setStyleId(cStyle.getId());
            Run run = new Run(TABLE_COLUMNS[i]);
            run.setTextFormat(txtFormat);
            cell.getRuns().add(run);
        }

        return ++nextRow;
    }

    private static int drawHeader(Table table, int nextRow, String headerText) throws Exception {

        // Create text format for the header
        TextFormat txtFormat = new TextFormat("Arial", (short) 14);
        txtFormat.setBold(true);

        // Little increase row height.
        Row row = table.getRow(nextRow);
        row.setHeight(row.getHeight() * 1.3f);

        // Create text block and add it to the cell
        Run run = new Run(headerText);
        run.setTextFormat(txtFormat);
        table.getCell(0, nextRow).getRuns().add(run);

        ++nextRow;

        // Create text format for the Date field
        txtFormat = new TextFormat("Arial", (short) 11);

        // Convert date to string
        DateFormat formatter = new SimpleDateFormat("MMMMM dd, yyyy", Locale.ENGLISH);
        String s = formatter.format(new Date());

        // Add date to the cell
        run = new Run(s);
        run.setTextFormat(txtFormat);
        table.getCell(0, nextRow).getRuns().add(run);

        nextRow += 2;
        return nextRow;
    }

    private static void AddChart(Document doc, Page page, int nextRow, String category,
            CategoryDataset dataset, String outputPath) throws IOException, DocumentException {

        // Create chart
        final JFreeChart chart = ChartFactory.createBarChart(
            "Sales by Category: " + category, // chart title
            "Product Name", // domain axis label
            "Product Sales", // range axis label
            dataset, // data
            PlotOrientation.HORIZONTAL, // the plot orientation
            false, // include legend
            true, false);

        CategoryPlot plot = (CategoryPlot) chart.getPlot();
        // Create and set custom renderer
        CustomRenderer renderer = new CustomRenderer();
        plot.setRenderer(renderer);

        // Save chart image to byte array.
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ChartUtilities.writeChartAsPNG(baos, chart, 500, 300);
        byte[] imageData = baos.toByteArray();

        // Create stream with image
        ByteArrayInputStream bais = new ByteArrayInputStream(imageData);

        // Create shape for image
        Picture img = Shape.createPicture();
        img.setData(bais);

        // Set the image bounds
        Rectangle2D bounds = img.getBounds();
        double x = 400;
        double y = 0;
        double w = 350;
        double h = 200;
        bounds.setRect(x, y, w, h);

        // AddShape to report page
        page.addShape(img);
    }
}

/**
 * Custom renderer Uses predefined colors from array to draw bars
 */
class CustomRenderer extends BarRenderer {

    private static final long serialVersionUID = -1277953692223717139L;

    /** The colors for items. */
    private Paint[] colors = new Paint[] {
            Color.red, Color.blue, Color.green, Color.yellow,
            Color.orange, Color.cyan, Color.magenta, Color.blue
    };

    /**
     * Creates a new renderer.
     */
    public CustomRenderer() {
    }

    /**
     * Returns the paint for an item. Overrides the default behavior inherited
     * from AbstractSeriesRenderer.
     * 
     * @param row
     *            the series.
     * @param column
     *            the category.
     * 
     * @return The item color.
     */
    public Paint getItemPaint(final int row, final int column) {
        return this.colors[column % this.colors.length];
    }
}
